==> org-lsst-ccs-subsystem-power-main-1.6.16-sources.jar: org/lsst/ccs/subsystem/power/QuadBox.java <==
package org.lsst.ccs.subsystem.power;

import java.time.Duration;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.ccs.Subsystem;
import org.lsst.ccs.bus.data.AgentInfo;
import org.lsst.ccs.bus.data.KeyValueData;
import org.lsst.ccs.command.annotations.Argument;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.command.annotations.Command.CommandType;
import org.lsst.ccs.commons.annotations.LookupField;
import org.lsst.ccs.framework.HasLifecycle;
import org.lsst.ccs.services.AgentPeriodicTaskService;
import org.lsst.ccs.services.AgentPropertiesService;
import org.lsst.ccs.subsystem.power.constants.PowerAgentProperties;
import org.lsst.ccs.subsystem.power.constants.QuadBoxSwitches;
import org.lsst.ccs.subsystem.power.constants.SwitchState;
import org.lsst.ccs.subsystem.power.data.PowerException;
import org.lsst.ccs.subsystem.power.data.QuadBoxState;

/**
 *  Implements the Quad Box control subsystem.
 *
 *  @author Owen Saxton
 */
public class QuadBox extends Subsystem implements HasLifecycle {

    static class DummyDevice implements SwitchControl {
        @Override
        public int getSwitchDevice() { return 0; }
        @Override
        public Boolean isSwitchOn(int sw) { return null; }
        @Override
        public void switchOn(int sw) {}
        @Override
        public void switchOff(int sw) {}
    }

    private static final Map<Integer, String> typeMap = new HashMap<>();
    static {
        typeMap.put(QuadBoxSwitches.DEVC_BFR, "BFR");
        typeMap.put(QuadBoxSwitches.DEVC_PDU_5V, "5V PDU");
        typeMap.put(QuadBoxSwitches.DEVC_PDU_24VC, "24V clean PDU");
        typeMap.put(QuadBoxSwitches.DEVC_PDU_24VD, "24V dirty PDU");
        typeMap.put(QuadBoxSwitches.DEVC_PDU_48V, "48V PDU");
        typeMap.put(QuadBoxSwitches.DEVC_REB_BULK, "REB bulk PS");
    }

    private static final Logger LOG = Logger.getLogger(QuadBox.class.getName());
    
    @LookupField(strategy = LookupField.Strategy.TREE)
    private AgentPropertiesService aps;

    @LookupField(strategy = LookupField.Strategy.TREE)
    private AgentPeriodicTaskService apts;

    @LookupField(strategy = LookupField.Strategy.CHILDREN)
    private final List<SwitchControl> swDeviceList = new ArrayList<>();

    private final SwitchControl[] swDevices = new SwitchControl[QuadBoxSwitches.NUM_DEVICES];
    private final QuadBoxState qbState = new QuadBoxState();


    /**
     *  Constructor.
     */
    public QuadBox() {
        super("quadbox", AgentInfo.AgentType.WORKER);
    }


    /**
     *  Subsystem post-initialization
     */
    @Override
    public void postInit()
    {
        // Set a property to define that this Agent is a Quad Box controller.
        aps.setAgentProperty(PowerAgentProperties.QUAD_BOX_AGENT, getClass().getCanonicalName());

        // Checks power devices and control channels
        boolean error = false;
        for (SwitchControl sd : swDeviceList) {
            swDevices[sd.getSwitchDevice()] = sd;
        }
        for (int j = 0; j < swDevices.length; j++) {
            if (swDevices[j] == null) {
                LOG.log(Level.SEVERE, "The {0} device has not been specified", typeMap.get(j));
                //swDevices[j] = new DummyDevice();
                error = true;
            }
        }
        if (error) {
            throw new RuntimeException("Fatal initialization error");
        }
    }


    /**
     *  Starts the subsystem.
     *
     */
    @Override
    public void postStart()
    {
        updateState();
        publishState();        // For any GUIs
        LOG.info("Quad Box subsystem started");
    }


    /**
     *  Performs periodic power data broadcast.
     */
    public void periodicBroadcast()
    {
        /*
        if (pwrDevices.isEmpty()) return;
        ArrayList<PowerChanState> pState = new ArrayList<>();
        for (PowerDevice pDevc : pwrDevices.values()) {
            try {
                pState.addAll(pDevc.getPowerState());
            }
            catch (Exception e) {
            }
        }
        KeyValueData kvd = new KeyValueData(PowerChanState.KEY, pState);
        subsys.publishSubsystemDataOnStatusBus(kvd);
        */
    }


    /**
     *  Turns a switch on or off.
     *
     *  @param  sw  The switch number.
     *  @param  on  Whether to turn on or off
     *  @throws  PowerException
     */
    @Command(type=Command.CommandType.ACTION, description="Turn on/off a switch")
    public void setSwitchOn(@Argument(description="The switch number") int sw,
                            @Argument(description="Whether to turn on") boolean on) throws PowerException
    {
        int devNum = sw >> 8;
        int swNum = (byte)sw;  // Sign extend byte value
        if (devNum < 0 || devNum >= QuadBoxSwitches.NUM_DEVICES || swNum >= QuadBoxSwitches.NUM_SWITCHES[devNum]) {
            throw new PowerException("Invalid switch number: " + sw);
        }
        if (on) {
            swDevices[devNum].switchOn(swNum);
        }
        else {
            swDevices[devNum].switchOff(swNum);
        }
        updateState();
        publishState();
    }


    /**
     *  Turns a named switch on or off.
     *
     *  @param  name  The switch name.
     *  @param  on  Whether to turn on or off
     *  @throws  PowerException
     */
    @Command(type=Command.CommandType.ACTION, description="Turn on/off a named switch")
    public void setNamedSwitchOn(@Argument(description="The switch name") String name,
                                 @Argument(description="Whether to turn on") boolean on) throws PowerException
    {
        Integer sw = QuadBoxSwitches.switchNameToId.get(name);
        if (sw == null) {
            throw new PowerException("Invalid switch name: " + name);
        }
        setSwitchOn(sw, on);
    }


    /**
     *  Gets the list of switch names.
     * 
     *  @return  The list of names
     */
    @Command(type=Command.CommandType.QUERY, description="Get list of switch names")
    public List<String> getSwitchNames()
    {
        return new ArrayList<>(QuadBoxSwitches.switchNameToId.keySet());
    }


    /**
     *  Sets the update (tick) period.
     *
     *  @param  period  The update period (milliseconds) to set.
     */
    @Command(type=CommandType.ACTION, description="Set the update period")
    public void setUpdatePeriod(@Argument(name="period", description="The tick period (msecs)")
                                int period)
    {
        setTickPeriod(period);
        publishState();    // Must do this for GUI
    }


    /**
     *  Gets the full quad box state.
     *
     *  @return  The full quad box state
     *  @throws  PowerException
     */
    @Command(type=CommandType.QUERY, description="Get the full state")
    public QuadBoxState getFullState() throws PowerException
    {
        updateState();
        qbState.setTickMillis(getTickPeriod());
        return qbState;
    }


    /**
     *  Updates the quad box state.
     */
    private void updateState()
    {
        for (int devNum = 0; devNum < QuadBoxSwitches.NUM_DEVICES; devNum++) {
            for (int swNum = -1; swNum < QuadBoxSwitches.NUM_SWITCHES[devNum]; swNum++) {
                Boolean st = swDevices[devNum].isSwitchOn(swNum);
                SwitchState ss = st == null ? SwitchState.OFFLINE : st ? SwitchState.ON : SwitchState.OFF;
                qbState.setSwitchState(devNum, swNum, ss);
            }
        }
    }


    /**
     *  Sets the tick period
     */
    private void setTickPeriod(long period)
    {
        apts.setPeriodicTaskPeriod("monitor-publish", Duration.ofMillis(period));
    }
    

    /**
     *  Gets the tick period
     */
    private int getTickPeriod()
    {
        return (int)apts.getPeriodicTaskPeriod("monitor-publish").toMillis();
    }
    

    /**
     *  Publishes the state of the quad box system.
     *
     *  This is intended to be called whenever any element of the state is
     *  changed.
     */
    private void publishState()
    {
        qbState.setTickMillis(getTickPeriod());
        KeyValueData kvd = new KeyValueData(QuadBoxState.KEY, qbState);
        publishSubsystemDataOnStatusBus(kvd);
    }    

}

==> org-lsst-ccs-subsystem-power-main-1.6.25-sources.jar: org/lsst/ccs/subsystem/power/QuadBox.java <==
package org.lsst.ccs.subsystem.power;

import java.time.Duration;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lsst.ccs.Subsystem;
import org.lsst.ccs.bus.data.AgentInfo;
import org.lsst.ccs.bus.data.KeyValueData;
import org.lsst.ccs.command.annotations.Argument;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.command.annotations.Command.CommandType;
import org.lsst.ccs.commons.annotations.LookupField;
import org.lsst.ccs.framework.AgentPeriodicTask;
import org.lsst.ccs.framework.HasLifecycle;
import org.lsst.ccs.services.AgentPeriodicTaskService;
import org.lsst.ccs.services.AgentPropertiesService;
import org.lsst.ccs.subsystem.common.MonitorTaskControl;
import org.lsst.ccs.subsystem.power.constants.MonitorControl;
import org.lsst.ccs.subsystem.power.constants.PowerAgentProperties;
import org.lsst.ccs.subsystem.power.constants.QuadBoxSwitches;
import org.lsst.ccs.subsystem.power.constants.SwitchState;
import org.lsst.ccs.subsystem.power.data.PowerException;
import org.lsst.ccs.subsystem.power.data.QuadBoxState;

/**
 *  Implements the Quad Box control subsystem.
 *
 *  @author Owen Saxton
 */
public class QuadBox extends Subsystem implements HasLifecycle {

    static class DummyDevice implements SwitchControl {
        @Override
        public int getSwitchDevice() { return 0; }
        @Override
        public Boolean isSwitchOn(int sw) { return null; }
        @Override
        public void switchOn(int sw) {}
        @Override
        public void switchOff(int sw) {}
    }

    private static final Map<Integer, String> typeMap = new HashMap<>();
    static {
        typeMap.put(QuadBoxSwitches.DEVC_BFR, "BFR");
        typeMap.put(QuadBoxSwitches.DEVC_PDU_5V, "5V PDU");
        typeMap.put(QuadBoxSwitches.DEVC_PDU_24VC, "24V clean PDU");
        typeMap.put(QuadBoxSwitches.DEVC_PDU_24VD, "24V dirty PDU");
        typeMap.put(QuadBoxSwitches.DEVC_PDU_48V, "48V PDU");
        typeMap.put(QuadBoxSwitches.DEVC_REB_BULK, "REB bulk PS");
    }

    private static final Logger LOG = Logger.getLogger(QuadBox.class.getName());
    
    @LookupField(strategy = LookupField.Strategy.TREE)
    private AgentPropertiesService aps;
    @LookupField(strategy = LookupField.Strategy.TREE)
    private AgentPeriodicTaskService apts;
    @LookupField(strategy = LookupField.Strategy.CHILDREN)
    private final List<SwitchControl> swDeviceList = new ArrayList<>();

    private final SwitchControl[] swDevices = new SwitchControl[QuadBoxSwitches.NUM_DEVICES];
    private final QuadBoxState qbState = new QuadBoxState();
    private MonitorTaskControl monitorControl;


    /**
     *  Constructor.
     */
    public QuadBox() {
        super("quadbox", AgentInfo.AgentType.WORKER);
        getAgentInfo().getAgentProperties().setProperty("org.lsst.ccs.use.full.paths", "true");
    }


    /**
     *  Build phase
     */
    @Override
    public void build() {
        // Create the monitor task control object and node
        monitorControl = MonitorTaskControl.createNode(this, MonitorControl.NODE_NAME);

        // Create and schedule an AgentPeriodicTask to update and publish the utility trunk system state
        AgentPeriodicTask pt = new AgentPeriodicTask("quadbox-state", () -> updateState()).withPeriod(Duration.ofMillis(1000));
        apts.scheduleAgentPeriodicTask(pt);
    }


    /**
     *  Subsystem post-initialization
     */
    @Override
    public void postInit()
    {
        // Set a property to define that this Agent is a Quad Box controller.
        aps.setAgentProperty(PowerAgentProperties.QUAD_BOX_AGENT, getClass().getCanonicalName());

        // Checks power devices and control channels
        boolean error = false;
        for (SwitchControl sd : swDeviceList) {
            swDevices[sd.getSwitchDevice()] = sd;
        }
        for (int j = 0; j < swDevices.length; j++) {
            if (swDevices[j] == null) {
                LOG.log(Level.SEVERE, "The {0} device has not been specified", typeMap.get(j));
                //swDevices[j] = new DummyDevice();
                error = true;
            }
        }
        if (error) {
            throw new RuntimeException("Fatal initialization error");
        }
    }


    /**
     *  Starts the subsystem.
     *
     */
    @Override
    public void postStart()
    {
        publishState();        // For any GUIs
        LOG.info("Quad Box subsystem started");
    }


    /**
     *  Turns a switch on or off.
     *
     *  @param  name  The switch name.
     *  @param  on  Whether to turn on or off
     *  @throws  PowerException
     */
    @Command(type=Command.CommandType.ACTION, description="Turn on/off a named switch")
    public void setSwitchOn(@Argument(description="The switch name") String name,
                            @Argument(description="Whether to turn on") boolean on) throws PowerException
    {
        Integer sw = QuadBoxSwitches.getSwitchId(name);
        if (sw == null) {
            throw new PowerException("Invalid switch name: " + name);
        }
        int devNum = sw >> 8;
        int swNum = (byte)(int)sw;  // Sign extend byte value
        if (devNum < 0 || devNum >= QuadBoxSwitches.NUM_DEVICES || swNum >= QuadBoxSwitches.NUM_SWITCHES[devNum]) {
            throw new PowerException("Programming error!! Invalid switch number: " + sw);
        }
        if (on) {
            swDevices[devNum].switchOn(swNum);
        }
        else {
            swDevices[devNum].switchOff(swNum);
        }
    }


    /**
     *  Gets the list of switch names.
     * 
     *  @return  The list of names
     */
    @Command(type=Command.CommandType.QUERY, description="Get list of switch names")
    public List<String> getSwitchNames()
    {
        return QuadBoxSwitches.getSwitchNames();
    }


    /**
     *  Gets the full quad box state.
     *
     *  @return  The full quad box state
     */
    @Command(type=CommandType.QUERY, description="Get the full state")
    public QuadBoxState getFullState()
    {
        qbState.setTickMillis(monitorControl.getPublishPeriod());
        return qbState;
    }


    /**
     *  Updates the quad box state.
     */
    private void updateState()
    {
        boolean changed = monitorControl.hasPeriodChanged();
        for (int devNum = 0; devNum < QuadBoxSwitches.NUM_DEVICES; devNum++) {
            for (int swNum = -1; swNum < QuadBoxSwitches.NUM_SWITCHES[devNum]; swNum++) {
                Boolean st = swDevices[devNum].isSwitchOn(swNum);
                SwitchState ss = st == null ? SwitchState.OFFLINE : st ? SwitchState.ON : SwitchState.OFF;
                if (ss != qbState.getSwitchState(devNum, swNum)) {
                    qbState.setSwitchState(devNum, swNum, ss);
                    changed = true;
                }
            }
        }
        if (changed) {
            publishState();
        }
    }


    /**
     *  Publishes the state of the quad box system.
     *
     *  This is intended to be called whenever any element of the state is
     *  changed.
     */
    private void publishState()
    {
        KeyValueData kvd = new KeyValueData(QuadBoxState.KEY, getFullState());
        publishSubsystemDataOnStatusBus(kvd);
    }    

}
